@@ -2,7 +2,7 @@
  * dselect - Debian package maintenance user interface
  * pkgsublist.cc - status modification and recursive package list handling
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2007-2014 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -22,7 +22,6 @@
 #include <config.h>
 #include <compat.h>
 
-#include <assert.h>
 #include <string.h>
 #include <stdio.h>
 
@@ -76,7 +75,6 @@ void packagelist::add(pkginfo *pkg, const char *extrainfo, showpriority showimp)
   add(pkg);  if (!pkg->clientdata) return;
   if (pkg->clientdata->dpriority < showimp) pkg->clientdata->dpriority= showimp;
   pkg->clientdata->relations(extrainfo);
-  pkg->clientdata->relations.terminate();
 }
 
 bool
@@ -103,9 +101,15 @@ void packagelist::addunavailable(deppossi *possi) {
   if (!recursive) return;
   if (alreadydone(&unavdone,possi)) return;
 
-  assert(possi->up->up->clientdata);
-  assert(possi->up->up->clientdata->uprec);
+  if (possi->up->up->clientdata == nullptr)
+    internerr("deppossi from package %s has nullptr clientdata",
+              pkg_name(possi->up->up, pnaw_always));
+  if (possi->up->up->clientdata->uprec == nullptr)
+    internerr("deppossi from package %s has nullptr clientdata's uprec",
+              pkg_name(possi->up->up, pnaw_always));
 
+  // cppcheck-suppress[constVariable]: false positive, operator() modifies it.
+  // cppcheck-suppress[constVariableReference]: idem, new name for the same.
   varbuf& vb= possi->up->up->clientdata->relations;
   vb(possi->ed->name);
   vb(_(" does not appear to be available\n"));
@@ -120,45 +124,45 @@ packagelist::add(dependency *depends, showpriority displayimportance)
     return false;
 
   const char *comma= "";
-  varbuf info;
-  info(depends->up->set->name);
-  info(' ');
-  info(gettext(relatestrings[depends->type]));
-  info(' ');
+  varbuf depinfo;
+  depinfo(depends->up->set->name);
+  depinfo(' ');
+  depinfo(gettext(relatestrings[depends->type]));
+  depinfo(' ');
   deppossi *possi;
   for (possi=depends->list;
        possi;
        possi=possi->next, comma=(possi && possi->next ? ", " : _(" or "))) {
-    info(comma);
-    info(possi->ed->name);
+    depinfo(comma);
+    depinfo(possi->ed->name);
     if (possi->verrel != DPKG_RELATION_NONE) {
       switch (possi->verrel) {
       case DPKG_RELATION_LE:
-        info(" (<= ");
+        depinfo(" (<= ");
         break;
       case DPKG_RELATION_GE:
-        info(" (>= ");
+        depinfo(" (>= ");
         break;
       case DPKG_RELATION_LT:
-        info(" (<< ");
+        depinfo(" (<< ");
         break;
       case DPKG_RELATION_GT:
-        info(" (>> ");
+        depinfo(" (>> ");
         break;
       case DPKG_RELATION_EQ:
-        info(" (= ");
+        depinfo(" (= ");
         break;
       default:
         internerr("unknown dpkg_relation %d", possi->verrel);
       }
-      info(versiondescribe(&possi->version, vdew_nonambig));
-      info(")");
+      depinfo(versiondescribe(&possi->version, vdew_nonambig));
+      depinfo(")");
     }
   }
-  info('\n');
-  add(depends->up,info.string(),displayimportance);
+  depinfo('\n');
+  add(depends->up, depinfo.string(), displayimportance);
   for (possi=depends->list; possi; possi=possi->next) {
-    add(&possi->ed->pkg, info.string(), displayimportance);
+    add(&possi->ed->pkg, depinfo.string(), displayimportance);
     if (depends->type != dep_provides) {
       /* Providers are not relevant if we are looking at a provider
        * relationship already. */
@@ -167,7 +171,7 @@ packagelist::add(dependency *depends, showpriority displayimportance)
            provider;
            provider = provider->rev_next) {
         if (provider->up->type != dep_provides) continue;
-        add(provider->up->up,info.string(),displayimportance);
+        add(provider->up->up, depinfo.string(), displayimportance);
         add(provider->up,displayimportance);
       }
     }
@@ -178,14 +182,14 @@ packagelist::add(dependency *depends, showpriority displayimportance)
 void repeatedlydisplay(packagelist *sub,
                        showpriority initial,
                        packagelist *unredisplay) {
-  pkginfo **newl;
-  keybindings *kb;
-
   debug(dbg_general, "repeatedlydisplay(packagelist[%p])", sub);
   if (sub->resolvesuggest() != 0 && sub->deletelessimp_anyleft(initial)) {
     debug(dbg_general, "repeatedlydisplay(packagelist[%p]) once", sub);
     if (unredisplay) unredisplay->enddisplay();
     for (;;) {
+      pkginfo **newl;
+      keybindings *kb;
+
       /* Reset manual_install flag now that resolvesuggest() has seen it. */
       manual_install = false;
       newl= sub->display();
