@@ -80,7 +80,7 @@ struct trig_hooks {
 
 #define TRIGHOOKS_DEFINE_NAMENODE_ACCESSORS				 \
   static struct fsys_namenode *th_nn_find(const char *name, bool nonew)	 \
-    { return fsys_hash_find_node(name, nonew ? FHFF_NONE : 0); }		 \
+    { return fsys_hash_find_node(name, nonew ? FHFF_NONEW : 0); }	 \
   static struct trigfileint **th_nn_interested(struct fsys_namenode *fnn) \
     { return &fnn->trig_interested; }					 \
   static const char *th_nn_name(struct fsys_namenode *fnn)		 \
