@@ -13,12 +13,26 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Control::Fields;
+=encoding utf8
+
+=head1 NAME
+
+Dpkg::Control::Fields - manage (list of official) control fields
+
+=head1 DESCRIPTION
+
+The module contains a list of vendor-neutral and vendor-specific fieldnames
+with associated meta-data explaining in which type of control information
+they are allowed. The vendor-neutral fieldnames and all functions are
+inherited from Dpkg::Control::FieldsCore.
+
+=cut
+
+package Dpkg::Control::Fields 1.00;
 
 use strict;
 use warnings;
 
-our $VERSION = '1.00';
 our @EXPORT = @Dpkg::Control::FieldsCore::EXPORT;
 
 use Carp;
@@ -45,19 +59,6 @@ foreach my $op (run_vendor_hook('register-custom-fields')) {
     }
 }
 
-=encoding utf8
-
-=head1 NAME
-
-Dpkg::Control::Fields - manage (list of official) control fields
-
-=head1 DESCRIPTION
-
-The module contains a list of vendor-neutral and vendor-specific fieldnames
-with associated meta-data explaining in which type of control information
-they are allowed. The vendor-neutral fieldnames and all functions are
-inherited from Dpkg::Control::FieldsCore.
-
 =head1 CHANGES
 
 =head2 Version 1.00 (dpkg 1.15.6)
