@@ -1,4 +1,5 @@
 # Copyright © 2007-2010 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2009, 2012-2015 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,15 +12,26 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Control::Info;
+=encoding utf8
+
+=head1 NAME
+
+Dpkg::Control::Info - parse files like debian/control
+
+=head1 DESCRIPTION
+
+It provides a class to access data of files that follow the same
+syntax as F<debian/control>.
+
+=cut
+
+package Dpkg::Control::Info 1.01;
 
 use strict;
 use warnings;
 
-our $VERSION = '1.00';
-
 use Dpkg::Control;
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
@@ -29,41 +41,40 @@ use parent qw(Dpkg::Interface::Storable);
 use overload
     '@{}' => sub { return [ $_[0]->{source}, @{$_[0]->{packages}} ] };
 
-=encoding utf8
-
-=head1 NAME
-
-Dpkg::Control::Info - parse files like debian/control
-
-=head1 DESCRIPTION
-
-It provides an object to access data of files that follow the same
-syntax as F<debian/control>.
-
-=head1 FUNCTIONS
+=head1 METHODS
 
 =over 4
 
-=item $c = Dpkg::Control::Info->new($file)
+=item $c = Dpkg::Control::Info->new(%opts)
 
-Create a new Dpkg::Control::Info object for $file. If $file is omitted, it
-loads F<debian/control>. If file is "-", it parses the standard input.
+Create a new Dpkg::Control::Info object. Loads the file from the filename
+option, if no option is specified filename defaults to F<debian/control>.
+If a scalar is passed instead, it will be used as the filename. If filename
+is "-", it parses the standard input. If filename is undef no loading will
+be performed.
 
 =cut
 
 sub new {
-    my ($this, $arg) = @_;
+    my ($this, @args) = @_;
     my $class = ref($this) || $this;
     my $self = {
 	source => undef,
 	packages => [],
     };
     bless $self, $class;
-    if ($arg) {
-	$self->load($arg);
+
+    my %opts;
+    if (scalar @args == 0) {
+        $opts{filename} = 'debian/control';
+    } elsif (scalar @args == 1) {
+        $opts{filename} = $args[0];
     } else {
-	$self->load('debian/control');
+        %opts = @args;
     }
+
+    $self->load($opts{filename}) if $opts{filename};
+
     return $self;
 }
 
@@ -79,17 +90,12 @@ sub reset {
     $self->{packages} = [];
 }
 
-=item $c->load($file)
-
-Load the content of $file. Exits in case of errors. If file is "-", it
-loads from the standard input.
-
 =item $c->parse($fh, $description)
 
 Parse a control file from the given filehandle. Exits in case of errors.
 $description is used to describe the filehandle, ideally it's a filename
-or a description of where the data comes from. It's used in error
-messages.
+or a description of where the data comes from. It is used in error messages.
+The data in the object is reset before parsing new control files.
 
 =cut
 
@@ -100,22 +106,29 @@ sub parse {
     return if not $cdata->parse($fh, $desc);
     $self->{source} = $cdata;
     unless (exists $cdata->{Source}) {
-	syntaxerr($desc, _g('first block lacks a source field'));
+        $cdata->parse_error($desc, g_("first stanza lacks a '%s' field"),
+                            'Source');
     }
     while (1) {
 	$cdata = Dpkg::Control->new(type => CTRL_INFO_PKG);
         last if not $cdata->parse($fh, $desc);
 	push @{$self->{packages}}, $cdata;
 	unless (exists $cdata->{Package}) {
-	    syntaxerr($desc, _g("block lacks the '%s' field"), 'Package');
+	    $cdata->parse_error($desc, g_("stanza lacks the '%s' field"),
+	                        'Package');
 	}
 	unless (exists $cdata->{Architecture}) {
-	    syntaxerr($desc, _g("block lacks the '%s' field"), 'Architecture');
+	    $cdata->parse_error($desc, g_("stanza lacks the '%s' field"),
+	                        'Architecture');
 	}
-
     }
 }
 
+=item $c->load($file)
+
+Load the content of $file. Exits in case of errors. If file is "-", it
+loads from the standard input.
+
 =item $c->[0]
 
 =item $c->get_source()
@@ -169,9 +182,10 @@ sub get_packages {
     return @{$self->{packages}};
 }
 
-=item $c->output($filehandle)
+=item $str = $c->output([$fh])
 
-Dump the content into a filehandle.
+Return the content info into a string. If $fh is specified print it into
+the filehandle.
 
 =cut
 
@@ -180,7 +194,7 @@ sub output {
     my $str;
     $str .= $self->{source}->output($fh);
     foreach my $pkg (@{$self->{packages}}) {
-	print $fh "\n";
+	print { $fh } "\n" if defined $fh;
 	$str .= "\n" . $pkg->output($fh);
     }
     return $str;
@@ -198,9 +212,15 @@ information.
 
 =back
 
-=head1 AUTHOR
+=head1 CHANGES
 
-Raphaël Hertzog <hertzog@debian.org>.
+=head2 Version 1.01 (dpkg 1.18.0)
+
+New argument: The $c->new() constructor accepts an %opts argument.
+
+=head2 Version 1.00 (dpkg 1.15.6)
+
+Mark the module as public.
 
 =cut
 
